use std::sync::mpsc::{Receiver, Sender};

use super::super::init::config;
use super::TunData;
use std::collections::HashMap;
use std::error::Error;
use std::net::Ipv4Addr;
use std::sync::Arc;
use tun_tap::Iface;

fn tun_recv(
    iface: &Arc<Iface>,
    sender_vec: &Vec<Sender<TunData>>,
    ip_to_thread: &HashMap<Ipv4Addr, u32>,
) -> Result<(), Box<dyn Error>> {
    let mut buf = vec![0; super::TUN_PACKET_SIZE];
    let long = iface.recv(&mut buf)?;
    // min ipv4 header size
    if long > 24 {
        let ip_dest = Ipv4Addr::new(buf[20], buf[21], buf[22], buf[23]);
        let thread_id: usize = *ip_to_thread
            .get(&ip_dest)
            .ok_or(format!("no thread id was assigned to {}", &ip_dest))?
            as usize;
        sender_vec[thread_id].send(TunData { payload: buf })?;
    }

    Ok(())
}

pub fn tun_recv_thread(
    config: Arc<config::Config>,
    iface: Arc<Iface>,
    sender_vec: Vec<Sender<TunData>>,
) {
    let mut ip_to_thread: HashMap<Ipv4Addr, u32> = HashMap::new();
    for (i, elt) in config.peers.iter().enumerate() {
        ip_to_thread.insert(elt.ip, i as u32);
    }

    loop {
        match tun_recv(&iface, &sender_vec, &ip_to_thread) {
            Ok(_) => (),
            Err(e) => eprintln!("{}", e),
        }
    }
}

fn tun_send(iface: &Arc<Iface>, recv: &Receiver<TunData>) -> Result<(), Box<dyn Error>> {
    let payload = recv.recv()?.payload;
    iface.send(&payload).unwrap_or(0);
    Ok(())
}

pub fn tun_send_thread(iface: Arc<Iface>, recv: Receiver<TunData>) {
    loop {
        match tun_send(&iface, &recv) {
            Ok(_) => (),
            Err(e) => eprintln!("{}", e),
        }
    }
}
